/*

  File: SOPad.c

  Allows to call functions from SOPad.dll

*/

#ifdef __BORLANDC__
#pragma inline
#pragma hdrstop
#endif

#include <windows.h>
#include "SOPad.h"

/* Constants */
const char* SOPAD_LIBRARY_NAME = "sopad.dll";

/* Global variables */
HMODULE g_hSOPadLibrary = NULL;

/* Pointers to functions from DLL */
void* g_p_getPadSettings;
void* g_p_setPadSettings;
void* g_p_checkCom;
void* g_p_setCom;
void* g_p_setPadType;
void* g_p_getPID;
void* g_p_setPID;
void (*g_p_startRead)(void);
void* g_p_stopRead;
void* g_p_getBioData;
void (*g_p_clearBioData)(void);
char (*g_p_framesAvailable)(void);
void* g_p_getFrame;
void* g_p_registerCanvas;
void (*g_p_unRegisterCanvas)(void);
void* g_p_setLang;

/* Service functions */
BOOL SOPAD_Initialize()
{
	BOOL bResult;

	g_hSOPadLibrary = LoadLibrary(SOPAD_LIBRARY_NAME);

	if(g_hSOPadLibrary == NULL)
		return FALSE;

	g_p_getPadSettings = GetProcAddress(g_hSOPadLibrary, "getPadSettings");
	g_p_setPadSettings = GetProcAddress(g_hSOPadLibrary, "setPadSettings");
	g_p_checkCom = GetProcAddress(g_hSOPadLibrary, "checkCom");
	g_p_setCom = GetProcAddress(g_hSOPadLibrary, "setCom");
	g_p_setPadType = GetProcAddress(g_hSOPadLibrary, "setPadType");
	g_p_getPID = GetProcAddress(g_hSOPadLibrary, "getPID");
	g_p_setPID = GetProcAddress(g_hSOPadLibrary, "setPID");
	g_p_startRead = (void (*)(void)) GetProcAddress(g_hSOPadLibrary, "startRead");
	g_p_stopRead = GetProcAddress(g_hSOPadLibrary, "stopRead");
	g_p_getBioData = GetProcAddress(g_hSOPadLibrary, "getBioData");
	g_p_clearBioData = (void (*)(void)) GetProcAddress(g_hSOPadLibrary, "clearBioData");
	g_p_framesAvailable = (char (*)(void)) GetProcAddress(g_hSOPadLibrary, "framesAvailable");
	g_p_getFrame = GetProcAddress(g_hSOPadLibrary, "getFrame");
	g_p_registerCanvas = GetProcAddress(g_hSOPadLibrary, "registerCanvas");
	g_p_unRegisterCanvas = (void (*)(void)) GetProcAddress(g_hSOPadLibrary, "unRegisterCanvas");
	g_p_setLang = GetProcAddress(g_hSOPadLibrary, "setLang");

	bResult = g_p_getPadSettings && g_p_setPadSettings && g_p_checkCom && g_p_setCom &&
		g_p_setPadType && g_p_getPID && g_p_setPID && g_p_startRead && g_p_stopRead &&
		g_p_getBioData && g_p_clearBioData && g_p_framesAvailable && g_p_getFrame &&
		g_p_registerCanvas && g_p_unRegisterCanvas && g_p_setLang;

	if(!bResult)
	{
		FreeLibrary(g_hSOPadLibrary);
		g_hSOPadLibrary = NULL;
	}

	return bResult;
}

void SOPAD_Uninitialize()
{
	if(g_hSOPadLibrary != NULL)
	{
		FreeLibrary(g_hSOPadLibrary);
		g_hSOPadLibrary = NULL;
	}
}

/* Imported functions */
void CharPtrToString(char* pDest, const char* szSource)
{
	int len = strlen(szSource);
	*pDest = (char) len;
	memcpy(pDest + 1, szSource, len);
}

void StringToCharPtr(char* szDest, const char* pSource)
{
	int len = *pSource;
	memcpy(szDest, pSource + 1, len);
	szDest[len] = 0;
}

/* DLL functions */

void SOPAD_getPadSettings(const char* szOldSettings, const char* szCertStr,
						  char szNewSettings[SOPAD_MAX_STRING_LENGTH])
{
	char param1[SOPAD_MAX_STRING_LENGTH];
	char param2[SOPAD_MAX_STRING_LENGTH];
	char result[SOPAD_MAX_STRING_LENGTH] = { 0 };

	if(g_hSOPadLibrary == NULL)
		return;

	CharPtrToString(param1, szOldSettings);
	CharPtrToString(param2, szCertStr);

	__asm {
		lea		eax, param1;
		lea		edx, param2;
		lea		ecx, result;
		
		call	dword ptr g_p_getPadSettings;
	}

	StringToCharPtr(szNewSettings, result);
}

void SOPAD_setPadSettings(const char* szPadSettings, const char* szCertStr)
{
	char param1[SOPAD_MAX_STRING_LENGTH];
	char param2[SOPAD_MAX_STRING_LENGTH];

	if(g_hSOPadLibrary == NULL)
		return;

	CharPtrToString(param1, szPadSettings);
	CharPtrToString(param2, szCertStr);

	__asm {
		lea		eax, param1;
		lea		edx, param2;
		
		call	dword ptr g_p_setPadSettings;
	}
}

void SOPAD_checkCom(unsigned char nComPort, char szPadType[SOPAD_MAX_STRING_LENGTH])
{
	char param2[SOPAD_MAX_STRING_LENGTH];

	if(g_hSOPadLibrary == NULL)
		return;

	CharPtrToString(param2, szPadType);

	__asm {
		xor		eax, eax
		mov		al, nComPort;
		lea		edx, param2;
		mov		ecx, SOPAD_MAX_STRING_LENGTH - 1

		call	dword ptr g_p_checkCom;
	}

	StringToCharPtr(szPadType, param2);
}

void SOPAD_setCom(unsigned char nComPort)
{
	if(g_hSOPadLibrary == NULL)
		return;

	__asm {
		xor		eax, eax
		mov		al, nComPort;

		call	dword ptr g_p_setCom;
	}
}

void SOPAD_setPadType(const char* szPadType)
{
	char param1[SOPAD_MAX_STRING_LENGTH];

	if(g_hSOPadLibrary == NULL)
		return;

	CharPtrToString(param1, szPadType);

	__asm {
		lea		eax, param1

		call	dword ptr g_p_setCom;
	}
}

void SOPAD_getPID(SOPAD_PIDType pPadID, const char* szCertStr)
{
	char param2[SOPAD_MAX_STRING_LENGTH];

	if(g_hSOPadLibrary == NULL)
		return;

	CharPtrToString(param2, szCertStr);

	__asm {
		mov		eax, pPadID
		lea		edx, param2

		call	dword ptr g_p_getPID;
	}
}

void SOPAD_setPID(const SOPAD_PIDType pPadID, const char* szCertStr)
{
	char param2[SOPAD_MAX_STRING_LENGTH];

	if(g_hSOPadLibrary == NULL)
		return;

	CharPtrToString(param2, szCertStr);

	__asm {
		mov		eax, pPadID
		lea		edx, param2

		call	dword ptr g_p_setPID;
	}
}

void SOPAD_startRead()
{
	if(g_hSOPadLibrary == NULL)
		return;

	g_p_startRead();
}

void SOPAD_stopRead(SOPAD_PIDType pPadID, SOPAD_GMTStampType* pTimeStamp)
{
	if(g_hSOPadLibrary == NULL)
		return;

	__asm {
		mov		eax, pPadID
		mov		edx, pTimeStamp

		call	dword ptr g_p_stopRead;
	}
}

void SOPAD_clearBioData()
{
	if(g_hSOPadLibrary == NULL)
		return;

	g_p_clearBioData();
}

BOOL SOPAD_framesAvailable()
{
	if(g_hSOPadLibrary == NULL)
		return FALSE;

	return g_p_framesAvailable();
}

void SOPAD_getFrame(SOPAD_TDataFrame* pFrame)
{
	if(g_hSOPadLibrary == NULL)
		return;

	__asm {
		mov		eax, pFrame

		call	dword ptr g_p_getFrame;
	}
}

void SOPAD_registerCanvas(void* pCanvas, int xoffs, int yoffs, int width, int height)
{
	if(g_hSOPadLibrary == NULL)
		return;

	__asm {
		mov		eax, pCanvas;
		mov		edx, xoffs;
		mov		ecx, yoffs;
		push	width;
		push	height;

		call	dword ptr g_p_registerCanvas;
	}
}

void SOPAD_unRegisterCanvas()
{
	if(g_hSOPadLibrary == NULL)
		return;

	g_p_unRegisterCanvas();
}

void SOPAD_setLang(const char* szLang)
{
	char param1[SOPAD_MAX_STRING_LENGTH];

	if(g_hSOPadLibrary == NULL)
		return;

	CharPtrToString(param1, szLang);

	__asm {
		lea		eax, param1

		call	dword ptr g_p_setLang;
	}
}