#include <windows.h>
#include <stdio.h>

#include "SOPad.h"
#include "spdsrv01.h"

/* Forward declarations */
void getSettings();
void setSettings();
void checkCom();
void setCom();
void setPadType();
void getPID();
void setPID();
void stopRead();
void framesAvailable();
void getFrame();

void TestCanvas();


int WINAPI WinMain(HINSTANCE hInstace, HINSTANCE hPrevInstance,
					   LPSTR lpCmdLine, int nShowCmd)
{
	if(SOPAD_Initialize())
	{
/*
		getSettings();
		setSettings();
		checkCom();
		setCom();
		setPadType();
		getPID();
		setPID();
		SOPAD_startRead();
		stopRead();
		SOPAD_clearBioData();
		framesAvailable();
		SOPAD_unRegisterCanvas();
		SOPAD_setLang("D");
		getFrame();
*/
		if(SRV_Initialize())
		{
			TestCanvas();
			SRV_Uninitialize();
		}

		SOPAD_Uninitialize();
	}
}

void setSettings()
{
	const char* szSettings = "000000000000000002no pad";
	const char* szCertStr = "";

	SOPAD_setPadSettings(szSettings, szCertStr);
}

void getSettings()
{
	const char* szOldSettings = "000000000000000000no pad";
	const char* szCertStr = "";
	char szNewSettings[SOPAD_MAX_STRING_LENGTH];

	SOPAD_getPadSettings(szOldSettings, szCertStr, szNewSettings);

	MessageBox(NULL, szNewSettings, "getPadSettings result", MB_OK | MB_ICONINFORMATION);
}

void checkCom()
{
	char szPadType[SOPAD_MAX_STRING_LENGTH] = { 0 };

	SOPAD_checkCom(1, szPadType);

	MessageBox(NULL, szPadType, "checkCom result", MB_OK | MB_ICONINFORMATION);
}

void setCom()
{
	SOPAD_setCom(1);
}

void setPadType()
{
	const char* szPadType = "bluePad";

	SOPAD_setPadType(szPadType);
}

void getPID()
{
	SOPAD_PIDType padID = { 0 };
	char szMessage[100];

	SOPAD_getPID(padID, "");

	sprintf(szMessage, "byte #0 = %i\nbyte #1 = %i\nbyte #2 = %i\nbyte #3 = %i\nbyte #4 = %i",
		(int) padID[0], (int) padID[1], (int) padID[2], (int) padID[3], (int) padID[4]);

	MessageBox(NULL, szMessage, "getPID result", MB_OK | MB_ICONINFORMATION);
}

void setPID()
{
	SOPAD_PIDType padID = { 0x55 };
	SOPAD_setPID(padID, "");
}

void stopRead()
{
	SOPAD_PIDType padID = { 0 };
	SOPAD_GMTStampType timeStamp;
	char szMessage[100];

	SOPAD_stopRead(padID, &timeStamp);

	sprintf(szMessage, "years = %i\nmonths = %i\ndays = %i\nhours = %i\nminutes = %i\nseconds = %i",
		(int) timeStamp.years, (int) timeStamp.months, (int) timeStamp.days,
		(int) timeStamp.hours, (int) timeStamp.minutes, (int) timeStamp.seconds);

	MessageBox(NULL, szMessage, "stopRead result", MB_OK | MB_ICONINFORMATION);
}

void framesAvailable()
{
	char szMessage[100];
	BOOL bAvailable = SOPAD_framesAvailable();

	sprintf(szMessage, "Result = %s", bAvailable ? "TRUE" : "FALSE");
	MessageBox(NULL, szMessage, "framesAvailable result", MB_OK | MB_ICONINFORMATION);
}

void getFrame()
{
	SOPAD_TDataFrame frame;
	char szMessage[100];

	/* Clear TDataFrame structure */
	memset(&frame, 0, sizeof frame);
	
	SOPAD_getFrame(&frame);
	
	sprintf(szMessage, "x = %i\ny = %i\nz = %i\nFrameNumber = %i\nLineNumber = %i\ntimecode = %i\n",
		(int) frame.x, (int) frame.y, (int) frame.z, frame.FrameNumber,
		(int) frame.LineNumber, frame.timecode);
	
	MessageBox(NULL, szMessage, "getFrame result", MB_OK | MB_ICONINFORMATION);
}

void TestCanvas()
{
	HWND hwnd;
	HDC hdc;
	void* pCanvas;
	
	hwnd = GetDesktopWindow();
	hdc = GetDC(hwnd);

	pCanvas = SRV_CreateCanvas(hdc);
	SOPAD_registerCanvas(pCanvas, 0, 0, 100, 100);
	SOPAD_unRegisterCanvas();
	SRV_DeleteCanvas(pCanvas);
	
	ReleaseDC(hwnd, hdc);
}