/*

  File: spdsrv01.c

  Calls spdsrv01.dll to create and delete TCanvas objects

*/

#include <windows.h>
#include "spdsrv01.h"

/* Constants */
const char* SRV_LIBRARY_NAME = "spdsrv01.dll";

/* Global variables */
HMODULE g_hSRVLibrary = NULL;

/* Pointers to functions from DLL */
void* (__stdcall *g_p_CreateCanvas)(HDC hdc);
void (__stdcall *g_p_DeleteCanvas)(void* pCanvas);

/* Service functions */
BOOL SRV_Initialize()
{
	BOOL bResult;

	g_hSRVLibrary = LoadLibrary(SRV_LIBRARY_NAME);

	if(g_hSRVLibrary == NULL)
		return FALSE;

	g_p_CreateCanvas = (void* (__stdcall *)(HDC)) GetProcAddress(g_hSRVLibrary, "CreateCanvas");
	g_p_DeleteCanvas = (void (__stdcall *)(void*)) GetProcAddress(g_hSRVLibrary, "DeleteCanvas");

	bResult = g_p_CreateCanvas && g_p_DeleteCanvas;

	if(!bResult)
	{
		FreeLibrary(g_hSRVLibrary);
		g_hSRVLibrary = NULL;
	}

	return bResult;
}

void SRV_Uninitialize()
{
	if(g_hSRVLibrary != NULL)
	{
		FreeLibrary(g_hSRVLibrary);
		g_hSRVLibrary = NULL;
	}
}

/* Imported functions */
void* SRV_CreateCanvas(HDC hdc)
{
	if(g_hSRVLibrary == NULL)
		return NULL;

	return g_p_CreateCanvas(hdc);
}

void SRV_DeleteCanvas(void* pCanvas)
{
	if(g_hSRVLibrary == NULL)
		return;

	g_p_DeleteCanvas(pCanvas);
}